'use strict';

/**
 * Request
 * @param  {Object} options       Request options.
 * @param  {String} options.type  Request type/method.
 * @param  {String} options.url   URL.
 * @param  {String} options.data  Post/put data.
 */

function request(options) {
  return new Promise(function (resolve, reject) {
    options.type = options.type || 'GET';

    var req = new XMLHttpRequest();

    req.open(options.type, options.url, true);

    req.onload = function () {
      if (req.status >= 200 && req.status < 400) {
        resolve(req.responseText);
      } else {
        reject(new Error(req.responseText));
      }
    };

    req.onerror = function () {
      reject(req.responseText);
    };

    if (options.data) {
      req.setRequestHeader('Content-Type', 'application/json; charset=UTF-8');
    }

    req.send(JSON.stringify(options.data));
  });
}

exports.request = request;
//# sourceMappingURL=request.js.map