'use strict';

var session = require('./lib/session');
var Variation = require('./variation');

/**
 * Test
 * @param {Object}        options             Test parameters.
 * @param {String|Number} options.id          Test id.
 * @param {Array}         options.variations  Variations.
 */
function Test(options) {
  if (!options) {
    throw new Error('Missing test options object. Test options required.');
  }

  if (typeof options.id === 'undefined') {
    throw new Error('Missing test ID. Test ID required.');
  }

  this.id = options.id;
  this.selectedVariation = null;
  this.variations = [];

  if (options.variations) {
    this.addVariations(options.variations);
  }

  this.loadFromSession();
  this.saveReference();
}

Test.tests = {};

Test.prototype.addVariations = function (variations) {
  return variations.map(function (variation) {
    return this.addVariation(variation);
  }, this);
};

Test.prototype.addVariation = function (variation) {
  if (variation instanceof Variation === false) {
    variation = new Variation(variation);
  }
  this.variations.push(variation);
  return variation;
};

Test.prototype.removeVariation = function (variation) {
  return this.variations.splice(this.variations.indexOf(variation), 1);
};

Test.prototype.selectVariation = function () {
  var variations = this.variations;

  var total = variations.reduce(function (previous, current) {
    return previous.rate + current.rate;
  });

  for (var i = 0, n = variations.length; i < n; i++) {
    var variation = variations[i];
    var rate = variation.rate / total;
    if (Math.random() <= rate) {
      return variation;
    }
  }

  return variations[variations.length - 1];
};

Test.prototype.run = function (data) {
  this.selectedVariation = this.selectedVariation || this.selectVariation();
  this.saveToSession();
  return this.selectedVariation.run(data);
};

Test.prototype.saveReference = function () {
  Test.tests[this.id] = this;
};

Test.prototype.loadFromSession = function () {
  var data = session.getTest(this.id);
  if (data) {
    this.selectedVariation = this.variations.filter(function (variation) {
      return variation.id === data.selectedVariation.id;
    })[0];
  }
};

Test.prototype.saveToSession = function () {
  var data = {
    selectedVariation: {
      id: this.selectedVariation.id
    }
  };
  session.saveTest(this.id, data);
};

module.exports = Test;
//# sourceMappingURL=test.js.map