'use strict';

var _require = require('./lib/request');

var request = _require.request;

/**
 * Variation
 * @param {Object}        options             Variation options.
 * @param {String|Number} options.id          Variation id.
 * @param {Number}        options.rate        Variation rate.
 * @param {Function}      options.fn          Variation function.
 */

function Variation(options) {
  if (!options) {
    throw new Error('Missing variation options object. Variation options required.');
  }

  if (typeof options.id === 'undefined') {
    throw new Error('Missing variation ID. Variation ID required.');
  }

  if (typeof options.fn === 'undefined') {
    throw new Error('Missing variation function. Variation function required.');
  }

  this.id = options.id;
  this.rate = options.rate || 0.1;
  this.fn = options.fn;

  this.recordView = this.recordView.bind(this);
  this.recordSuccess = this.recordSuccess.bind(this);
  this.recordFailure = this.recordFailure.bind(this);

  this.saveReference();
}

Variation.host = 'https://www.goabbey.top';
Variation.variations = {};

Variation.prototype.recordView = function (data) {
  return request({
    type: 'POST',
    url: Variation.host + '/api/hits',
    data: {
      variationId: this.id,
      type: 'view',
      data: data
    }
  }).catch(function (err) {
    console.error(err);
    throw err;
  });
};

Variation.prototype.recordSuccess = function (data) {
  return request({
    type: 'POST',
    url: Variation.host + '/api/hits',
    data: {
      variationId: this.id,
      type: 'success',
      data: data
    }
  }).catch(function (err) {
    console.error(err);
    throw err;
  });
};

Variation.prototype.recordFailure = function (data) {
  return request({
    type: 'POST',
    url: Variation.host + '/api/hits',
    data: {
      variationId: this.id,
      type: 'failure',
      data: data
    }
  }).catch(function (err) {
    console.error(err);
    throw err;
  });
};

Variation.prototype.run = function (data) {
  this.recordView(data);
  return this.fn.call(this, this.recordSuccess, this.recordFailure) || Promise.resolve(this);
};

Variation.prototype.saveReference = function () {
  Variation.variations[this.id] = this;
};

module.exports = Variation;
//# sourceMappingURL=variation.js.map